use crate::{gdev, Client};

use anyhow::Result;
use sp_core::{crypto::AccountId32, sr25519::Pair};
use subxt::tx::{BaseExtrinsicParamsBuilder, PairSigner};

type Call = gdev::runtime_types::gdev_runtime::Call;
type BalancesCall = gdev::runtime_types::pallet_balances::pallet::Call;

pub async fn transfer(
	pair: Pair,
	client: Client,
	balance: u64,
	dest: AccountId32,
	keep_alive: bool,
) -> Result<()> {
	if keep_alive {
		client
			.tx()
			.sign_and_submit_then_watch(
				&gdev::tx().balances().transfer(dest.into(), balance),
				&PairSigner::new(pair),
				BaseExtrinsicParamsBuilder::new(),
			)
			.await?;
	} else {
		client
			.tx()
			.sign_and_submit_then_watch(
				&gdev::tx()
					.balances()
					.transfer_keep_alive(dest.into(), balance),
				&PairSigner::new(pair),
				BaseExtrinsicParamsBuilder::new(),
			)
			.await?;
	}

	Ok(())
}

pub async fn transfer_multiple(
	pair: Pair,
	client: Client,
	amount: u64,
	dests: Vec<AccountId32>,
) -> Result<()> {
	// build the list of transactions from the destination accounts
	let transactions: Vec<Call> = dests
		.into_iter()
		.map(|dest| {
			Call::Balances(BalancesCall::transfer_keep_alive {
				dest: dest.into(),
				value: amount,
			})
		})
		.collect();

	// wrap these calls in a batch call
	client
		.tx()
		.sign_and_submit_then_watch(
			&gdev::tx().utility().batch(transactions),
			&PairSigner::new(pair.clone()),
			BaseExtrinsicParamsBuilder::new(),
		)
		.await?;

	Ok(())
}
